"use strict";
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var Parser = require("./Parsinator");
var chai_1 = require("chai");
suite("str", function () {
    var parser = Parser.str("Hello");
    test("hit", function () {
        chai_1.assert.strictEqual("Hello", Parser.run(parser, "Hello"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(parser, "Goodbye"); }, /Parse failure at 1:1: "Hello" not found/);
    });
});
suite("regex", function () {
    var parser = Parser.regex(/Hello, .*!/);
    test("hit", function () {
        chai_1.assert.strictEqual("Hello, world!", Parser.run(parser, "Hello, world!"));
        chai_1.assert.strictEqual("Hello, there!", Parser.run(parser, "Hello, there!"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(parser, "Oh! Hello, human!"); }, /Parse failure at 1:1: regex \/Hello, \.\*!\/ doesn't match/);
        chai_1.assert.throws(function () { return Parser.run(parser, "Goodbye, world!"); }, /Parse failure at 1:1: regex \/Hello, \.\*!\/ doesn't match/);
    });
});
suite("regexMatch", function () {
    var parser = Parser.regexMatch(/Hello, (.*)! ([0-9]+)/);
    test("hit", function () {
        chai_1.assert.deepEqual(["Hello, world! 123", "world", "123"], Parser.run(parser, "Hello, world! 123"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(parser, "Oh! Hello, human!"); }, /Parse failure at 1:1: regex \/Hello, \(\.\*\)! \(\[0-9\]\+\)\/ doesn't match/);
        chai_1.assert.throws(function () { return Parser.run(parser, "Goodbye, world!"); }, /Parse failure at 1:1: regex \/Hello, \(\.\*\)! \(\[0-9\]\+\)\/ doesn't match/);
    });
});
suite("maybe", function () {
    var parser = Parser.maybe(Parser.str("Hit"));
    test("hit", function () {
        chai_1.assert.strictEqual("Hit", Parser.run(parser, "Hit"));
    });
    test("miss", function () {
        chai_1.assert.isNull(Parser.run(parser, "Not a hit"));
    });
});
suite("many", function () {
    var parser = Parser.many(Parser.str("hi"));
    test("hit", function () {
        chai_1.assert.sameMembers(["hi", "hi", "hi", "hi"], Parser.run(parser, "hihihihi"));
    });
    test("miss succeeds with empty match", function () {
        chai_1.assert.sameMembers([], Parser.run(parser, "nope"));
    });
});
suite("many1", function () {
    var parser = Parser.many1(Parser.str("hi"));
    test("hit", function () {
        chai_1.assert.sameMembers(["hi", "hi", "hi", "hi"], Parser.run(parser, "hihihihi"));
    });
    test("miss fails", function () {
        chai_1.assert.throws(function () { return Parser.run(parser, "nope"); }, /Parse failure at 1:1: "hi" not found/);
    });
});
suite("choice", function () {
    var choice = Parser.choice([
        Parser.str("foo"),
        Parser.str("bar")
    ]);
    test("hit", function () {
        chai_1.assert.strictEqual("foo", Parser.run(choice, "foo"));
        chai_1.assert.strictEqual("bar", Parser.run(choice, "bar"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(choice, "baz"); }, /"foo" not found/);
        chai_1.assert.throws(function () { return Parser.run(choice, "baz"); }, /"bar" not found/);
    });
});
suite("sequence", function () {
    var parser = Parser.sequence([
        Parser.str("foo"),
        Parser.many(Parser.str("bar")),
        Parser.str("baz")
    ]);
    test("hit", function () {
        chai_1.assert.deepEqual(["foo", [], "baz"], Parser.run(parser, "foobaz"));
        chai_1.assert.deepEqual(["foo", ["bar"], "baz"], Parser.run(parser, "foobarbaz"));
        chai_1.assert.deepEqual(["foo", ["bar", "bar", "bar"], "baz"], Parser.run(parser, "foobarbarbarbaz"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(parser, "foobarbar"); }, /Parse failure at 1:10: "baz" not found/);
    });
});
suite("count", function () {
    var parser = Parser.str("ok");
    test("hit", function () {
        chai_1.assert.deepEqual(["ok", "ok"], Parser.run(Parser.count(2, parser), "okokok"));
        chai_1.assert.deepEqual(["ok"], Parser.run(Parser.count(1, parser), "okokok"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(Parser.count(2, parser), "okno"); }, /Parse failure at 1:3: "ok" not found/);
    });
});
suite("end", function () {
    var parser = Parser.sequence([
        Parser.str("this is the end"),
        Parser.end
    ]);
    test("hit", function () {
        chai_1.assert.deepEqual(["this is the end", null], Parser.run(parser, "this is the end"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(parser, "this is the end, except it's not"); }, /Parse failure at 1:16: Not at end of string/);
    });
});
suite("peek", function () {
    var parser = Parser.sequence([
        Parser.str("foo"),
        Parser.peek(Parser.str("bar")),
        Parser.str("barbaz")
    ]);
    test("hit", function () {
        chai_1.assert.deepEqual(["foo", "bar", "barbaz"], Parser.run(parser, "foobarbaz"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(parser, "foobazbaz"); }, /Parse failure at 1:4: "bar" not found/);
        chai_1.assert.throws(function () { return Parser.run(parser, "foobarbut"); }, /Parse failure at 1:4: "barbaz" not found/);
    });
});
suite("sepBy", function () {
    var words = Parser.sepBy(Parser.regex(/\s+/), Parser.regex(/[a-z]+/));
    test("hit", function () {
        chai_1.assert.deepEqual(["why", "hello", "there"], Parser.run(words, "why hello\nthere"));
        chai_1.assert.deepEqual(["why", "hello"], Parser.run(words, "why\thello"));
        chai_1.assert.deepEqual(["why"], Parser.run(words, "why"));
        chai_1.assert.deepEqual([], Parser.run(words, "123"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(words, "why hello 123"); }, /Parse failure at 1:11: regex \/\[a-z\]\+\/ doesn't match/);
    });
});
suite("sepBy1", function () {
    var words = Parser.sepBy1(Parser.regex(/\s+/), Parser.regex(/[a-z]+/));
    test("hit", function () {
        chai_1.assert.deepEqual(["why", "hello", "there"], Parser.run(words, "why hello\nthere"));
        chai_1.assert.deepEqual(["why", "hello"], Parser.run(words, "why\thello"));
        chai_1.assert.deepEqual(["why"], Parser.run(words, "why"));
    });
    test("miss", function () {
        chai_1.assert.throws(function () { return Parser.run(words, "123"); }, /Parse failure at 1:1: regex \/\[a-z\]\+\/ doesn't match/);
    });
});
suite("fromGenerator", function () {
    var parseDivisionExpression = Parser.fromGenerator(function () {
        var numerator, _a, denominator, _b;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    _a = parseFloat;
                    return [4 /*yield*/, Parser.regex(/[0-9]+\.[0-9]+/)];
                case 1:
                    numerator = _a.apply(void 0, [_c.sent()]);
                    return [4 /*yield*/, Parser.regex(/\s*/)];
                case 2:
                    _c.sent();
                    return [4 /*yield*/, Parser.str("/")];
                case 3:
                    _c.sent();
                    return [4 /*yield*/, Parser.regex(/\s*/)];
                case 4:
                    _c.sent();
                    _b = parseFloat;
                    return [4 /*yield*/, Parser.regex(/[0-9]+\.[0-9]+/)];
                case 5:
                    denominator = _b.apply(void 0, [_c.sent()]);
                    return [2 /*return*/, numerator / denominator];
            }
        });
    });
    test("can be used to parse a complex expression", function () {
        chai_1.assert.equal(3.0 / 5.0, Parser.run(parseDivisionExpression, "3.0 / 5.0"));
        chai_1.assert.equal(42.0 / 21.0, Parser.run(parseDivisionExpression, "42.0/21.0"));
        chai_1.assert.equal(3.0 / 5.0, Parser.run(parseDivisionExpression, "3.0     /       5.0"));
    });
});
suite("map", function () {
    test('basic wrapping', function () {
        var parser = Parser.str('hello');
        var excited = Parser.map(parser, function (str) { return str + '!'; });
        chai_1.assert.equal('hello!', Parser.run(excited, 'hello'));
    });
});
suite("Documentation", function () {
    test("homepage email string", function () {
        var emailParser = Parser.between(Parser.str("<"), Parser.str(">"));
        var urlParser = Parser.between(Parser.str("("), Parser.str(")"));
        var infoParser = Parser.fromGenerator(function () {
            var name, email, url;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Parser.until(Parser.choice([
                            Parser.str("<"),
                            Parser.str("("),
                            Parser.end
                        ]))];
                    case 1:
                        name = _a.sent();
                        return [4 /*yield*/, Parser.maybe(emailParser)];
                    case 2:
                        email = _a.sent();
                        return [4 /*yield*/, Parser.regex(/\s*/)];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, Parser.maybe(urlParser)];
                    case 4:
                        url = _a.sent();
                        return [4 /*yield*/, Parser.end];
                    case 5:
                        _a.sent();
                        return [2 /*return*/, {
                                name: name.trim(),
                                email: email,
                                url: url
                            }];
                }
            });
        });
        chai_1.assert.deepEqual({
            name: "Abba Cadabra",
            email: "abba@cadabra.com",
            url: "http://magic.website",
        }, Parser.run(infoParser, "Abba Cadabra <abba@cadabra.com> (http://magic.website)"));
        chai_1.assert.deepEqual({
            name: "Béla Bartók",
            email: null,
            url: "https://www.britannica.com/biography/Bela-Bartok",
        }, Parser.run(infoParser, "Béla Bartók (https://www.britannica.com/biography/Bela-Bartok)"));
        chai_1.assert.deepEqual({
            name: "马云",
            email: "jack@1688.com",
            url: null,
        }, Parser.run(infoParser, "马云 <jack@1688.com>"));
        chai_1.assert.deepEqual({
            name: "Parsinator",
            email: null,
            url: null
        }, Parser.run(infoParser, "Parsinator"));
    });
    test('readme es2015', function () {
        var parseNaturalNumber = Parser.map(Parser.regex(/[1-9][0-9]*|0/), function (str) { return parseInt(str, 10); });
        var parseSum = Parser.fromGenerator(function () {
            var left, right;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, parseNaturalNumber];
                    case 1:
                        left = _a.sent();
                        return [4 /*yield*/, Parser.str("+")];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, parseNaturalNumber];
                    case 3:
                        right = _a.sent();
                        return [2 /*return*/, left + right];
                }
            });
        });
        chai_1.assert.strictEqual(579, Parser.run(parseSum, "123+456"));
        chai_1.assert.throws(function () { return Parser.run(parseSum, "23.5+92"); }, 'Parse failure at 1:3: "+" not found');
    });
    test('readme es5', function () {
        var parseNaturalNumber = Parser.map(Parser.regex(/[1-9][0-9]*|0/), function (str) {
            return parseInt(str, 10);
        });
        var parseSum = Parser.map(Parser.sequence([
            parseNaturalNumber,
            Parser.str("+"),
            parseNaturalNumber
        ]), function (items) {
            return items[0] + items[2];
        });
        chai_1.assert.strictEqual(579, Parser.run(parseSum, "123+456"));
        chai_1.assert.throws(function () { return Parser.run(parseSum, "23.5+92"); }, 'Parse failure at 1:3: "+" not found');
    });
});
